/* Copyright (C) 2000-2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   There are special exceptions to the terms and conditions of the GPL as it
   is applied to this software. View the full text of the exception in file
   EXCEPTIONS in the directory of this software distribution.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MYODBCUtil.h"

/*!
    \brief  Read data source from system information.

    \note   This will not replace existing values in pDataSource.
*/            
BOOL MYODBCUtilReadDataSource( MYODBCUTIL_DATASOURCE *pDataSource, LPCSTR pszDSN )
{
    char    szEntryNames[SQL_MAX_DSN_LENGTH * MYODBCUTIL_MAX_DSN_NAMES];
    char *  pszEntryName;
    char    szValue[4096];

    if ( !pszDSN || !(*pszDSN) )
        return FALSE;

#if defined(__APPLE__)    
    /*!
        \note   OSX

                SQLGetPrivateProfileString is the proper call and is available - but
                at this time it appears utterly broken. So we call an alternative
                instead. 
    */
    if ( GetPrivateProfileString( pszDSN, NULL, NULL, szEntryNames, sizeof( szEntryNames ) - 1, "odbc.ini" ) < 1 )
#else
    if ( SQLGetPrivateProfileString( pszDSN, NULL, "", szEntryNames, sizeof( szEntryNames ) - 1, "ODBC.INI" ) < 1 )
#endif
        return FALSE;

    /*!
        Scan result and return TRUE if we find a match.
    */
    pszEntryName = szEntryNames;
    while ( *pszEntryName )
    {
        *szValue = '\0';
#if defined(__APPLE__)    
        if ( GetPrivateProfileString( pszDSN, pszEntryName, NULL, szValue, sizeof( szValue ) - 1, "odbc.ini" ) > 0 )
#else
        if ( SQLGetPrivateProfileString( pszDSN, pszEntryName, "", szValue, sizeof( szValue ) - 1, "ODBC.INI" ) > 0 )
#endif
        {
            /*!
                ODBC RULE
                (SQLDriverConnect)

                The driver uses any information it retrieves from the system information 
                to augment the information passed to it in the connection string. If the 
                information in the system information duplicates information in the 
                connection string, the driver uses the information in the connection 
                string.
            */    
            if ( strcasecmp( pszEntryName, "DATABASE" ) == 0 )
            {
                if ( !pDataSource->pszDATABASE )
                    pDataSource->pszDATABASE = strdup( szValue ) ;
            }
            else if ( strcasecmp( pszEntryName, "DESCRIPTION" ) == 0 )
            {
                if ( !pDataSource->pszDESCRIPTION )
                    pDataSource->pszDESCRIPTION = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "DRIVER" ) == 0 )
            {    
                if ( !pDataSource->pszDriverFileName )
                    pDataSource->pszDriverFileName = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "OPTION" ) == 0 )
            {    
                if ( !pDataSource->pszOPTION )
                    pDataSource->pszOPTION = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "PASSWORD" ) == 0 )
            {    
                if ( !pDataSource->pszPASSWORD )
                    pDataSource->pszPASSWORD = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "PORT" ) == 0 )
            {    
                if ( !pDataSource->pszPORT )
                    pDataSource->pszPORT = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "SERVER" ) == 0 )
            {    
                if ( !pDataSource->pszSERVER )
                    pDataSource->pszSERVER = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "SOCKET" ) == 0 )
            {    
                if ( !pDataSource->pszSOCKET )
                    pDataSource->pszSOCKET = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "STMT" ) == 0 )
            {    
                if ( !pDataSource->pszSTMT )
                    pDataSource->pszSTMT = strdup( szValue );
            }
            else if ( strcasecmp( pszEntryName, "USER" ) == 0 )
            {    
                if ( !pDataSource->pszUSER )
                    pDataSource->pszUSER = strdup( szValue );
            }
            else
            {
                /* What the ? */
            }
        }
        pszEntryName += strlen( pszEntryName ) + 1;
    } /* while */

    return TRUE;
}

